<?php

namespace Drupal\config_help\Entity;

use Drupal\Core\Entity\EntityViewBuilder;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Theme\Registry;
use Drupal\Core\Utility\Token;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a help topic view builder.
 */
class HelpViewBuilder extends EntityViewBuilder {

  /**
   * The token replacement service class.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * The help topic storage instance.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $helpStorage;

  /**
   * Creates a new HelpViewBuilder.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type object.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager object.
   * @param \Drupal\Core\Utility\Token $token
   *   The token replacement service class.
   * @param \Drupal\Core\Theme\Registry $theme_registry
   *   The theme registry.
   */
  public function __construct(EntityTypeInterface $entity_type, EntityTypeManagerInterface $entity_type_manager, LanguageManagerInterface $language_manager, Token $token, Registry $theme_registry) {
    $this->entityTypeId = $entity_type->id();
    $this->entityType = $entity_type;
    $this->helpStorage = $entity_type_manager->getStorage('config_help');
    $this->languageManager = $language_manager;
    $this->token = $token;
    $this->themeRegistry = $theme_registry;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static($entity_type,
      $container->get('entity_type.manager'),
      $container->get('language_manager'),
      $container->get('token'),
      $container->get('theme.registry')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function viewMultiple(array $entities = [], $view_mode = 'full', $langcode = NULL) {
    $output = [];

    /** @var \Drupal\config_help\Entity\HelpTopicInterface[] $entities */
    foreach ($entities as $entity_id => $help_topic) {
      // Get the cache information and other build defaults.
      $build = $this->getBuildDefaults($help_topic, $view_mode);
      $build['#langcode'] = $langcode;
      $build['#title'] = $help_topic->label();

      // Preprocess the body for tokens entered with | instead of :.
      // See issue https://www.drupal.org/project/2369943/issues/2943974.
      $body = $help_topic->getBody();
      $matches = [];
      // Find things that look like tokens, but using | instead of :, for
      // example [foo|bar_baz|bang]. Allowable characters for the token pieces
      // are letters, numbers, underscores, plus . and - characters.
      if (preg_match_all('/\[([\w.\-]+\|)+[\w.\-]+\]/', $body, $matches)) {
        // Replace the | with : in each token that was found.
        $tokens = array_unique($matches[0]);
        $replacements = [];
        foreach ($tokens as $token) {
          $replacements[] = str_replace('|', ':', $token);
        }
        // Replace all the tokens in the string. This could have all been done
        // with one preg_replace instead of this whole if() statement, but the
        // code would have been very difficult to read.
        $body = str_replace($tokens, $replacements, $body);
      }

      // Add in the body, with token replacement.
      $bubbleable_metadata = new BubbleableMetadata();
      $build['#body'] = [
        '#type' => 'processed_text',
        '#text' => $this->token->replace($body, [], [], $bubbleable_metadata),
        '#format' => $help_topic->getBodyFormat(),
      ];
      $bubbleable_metadata->applyTo($build['#body']);

      $output[$entity_id] = $build;
    }

    return $output;
  }

}
