<?php

namespace Drupal\config_help\Plugin\HelpTopic;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\help\HelpTopicPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Wraps the help topic entity into a help topic plugin.
 */
class DerivedHelpTopicPlugin extends HelpTopicPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The entity this plugin is wrapping.
   *
   * @var \Drupal\config_help\Entity\HelpTopicInterface
   */
  protected $entity;

  /**
   * The help topic entity storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $helpTopicStorage;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a DerivedHelpTopicPlugin.
   *
   * @param string $plugin_id
   *   The ID for the plugin instance.
   * @param array $plugin_definition
   *   The plugin definition.
   * @param \Drupal\Core\Entity\EntityStorageInterface $help_topic_storage
   *   The help topic storage.
   */
  public function __construct(array $configuration, $plugin_id, array $plugin_definition, EntityStorageInterface $help_topic_storage, EntityTypeManagerInterface $entity_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->helpTopicStorage = $help_topic_storage;
    $this->entityTypeManager = $entity_manager;

    if (!isset($plugin_definition['metadata']['entity_id'])) {
      throw new PluginException($this->t('Missing entity ID in plugin definition'));
    }
    $entities = $help_topic_storage->loadMultiple([$plugin_definition['metadata']['entity_id']]);
    if (!$entities || !count($entities)) {
      throw new PluginException($this->t('Invalid entity ID in plugin definition'));
    }
    $entity = reset($entities);
    $this->entity = clone($entity);

    // The plugin manager does some manipulation on the related field after
    // loading definitions. So, override with what is in the plugin definition.
    $this->entity->setRelated($plugin_definition['related']);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager')->getStorage('config_help'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getLabel() {
    return $this->entity->label();
  }

  /**
   * {@inheritdoc}
   */
  public function getBody() {
    return $this->entityTypeManager->getViewBuilder('config_help')->view($this->entity, 'default');
  }

  /**
   * {@inheritdoc}
   */
  public function getBodyFormat() {
    return $this->entity->getBodyFormat();
  }

  /**
   * {@inheritdoc}
   */
  public function isTopLevel() {
    return $this->entity->isTopLevel();
  }

  /**
   * {@inheritdoc}
   */
  public function getRelated() {
    return $this->entity->getRelated();
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return $this->entity->getCacheMaxAge();
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    return $this->entity->getCacheContexts();
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    return $this->entity->getCacheTags();
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTagsForList() {
    $tags = parent::getCacheTagsForList();
    foreach ($this->entityTypeManager->getDefinition('config_help')->getListCacheTags() as $tag) {
      $tags[] = $tag;
    }

    return $tags;
  }

}
